"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCoverage = exports.disableCoverage = exports.addCoveragePCOV = exports.addCoverageXdebug = exports.checkXdebugError = void 0;
const utils = __importStar(require("./utils"));
const extensions = __importStar(require("./extensions"));
const config = __importStar(require("./config"));
async function checkXdebugError(extension, version) {
    if ((/^5\.[3-6]$|^7\.[0-1]$/.test(version) && extension == 'xdebug3') ||
        (/^8\.[0-9]$/.test(version) && extension == 'xdebug2')) {
        return extension + ' is not supported on PHP ' + version;
    }
    return '';
}
exports.checkXdebugError = checkXdebugError;
async function addCoverageXdebug(extension, version, os, pipe) {
    let script = '\n';
    const error = await checkXdebugError(extension, version);
    if (!error) {
        script +=
            (await extensions.addExtension(':pcov:false', version, os, true)) + pipe;
        extension = extension == 'xdebug3' ? 'xdebug' : extension;
        script +=
            (await extensions.addExtension(extension, version, os, true)) + pipe;
        script += await utils.setVariable('xdebug_version', 'php -r "echo phpversion(\'xdebug\');"', os);
        script +=
            (await utils.getCommand(os, 'extension_log')) +
                'xdebug "Xdebug $xdebug_version enabled as coverage driver"';
    }
    else {
        script += await utils.addLog('$cross', extension, error, os);
    }
    return script;
}
exports.addCoverageXdebug = addCoverageXdebug;
async function addCoveragePCOV(version, os, pipe) {
    let script = '\n';
    switch (true) {
        default:
            script +=
                (await extensions.addExtension(':xdebug:false', version, os, true)) +
                    pipe;
            script +=
                (await extensions.addExtension('pcov', version, os, true)) + pipe;
            script += (await config.addINIValues('pcov.enabled=1', os, true)) + '\n';
            script += await utils.setVariable('pcov_version', 'php -r "echo phpversion(\'pcov\');"', os);
            script +=
                (await utils.getCommand(os, 'extension_log')) +
                    'pcov "PCOV $pcov_version enabled as coverage driver"';
            break;
        case /5\.[3-6]|7\.0/.test(version):
            script += await utils.addLog('$cross', 'pcov', 'PHP 7.1 or newer is required', os);
            break;
    }
    return script;
}
exports.addCoveragePCOV = addCoveragePCOV;
async function disableCoverage(version, os, pipe) {
    let script = '\n';
    script +=
        (await extensions.addExtension(':pcov:false', version, os, true)) + pipe;
    script +=
        (await extensions.addExtension(':xdebug:false', version, os, true)) + pipe;
    script += await utils.addLog('$tick', 'none', 'Disabled Xdebug and PCOV', os);
    return script;
}
exports.disableCoverage = disableCoverage;
async function addCoverage(coverage_driver, version, os) {
    coverage_driver = coverage_driver.toLowerCase();
    const script = '\n' + (await utils.stepLog('Setup Coverage', os));
    const pipe = (await utils.suppressOutput(os)) + '\n';
    switch (coverage_driver) {
        case 'pcov':
            return script + (await addCoveragePCOV(version, os, pipe));
        case 'xdebug':
        case 'xdebug2':
        case 'xdebug3':
            return (script + (await addCoverageXdebug(coverage_driver, version, os, pipe)));
        case 'none':
            return script + (await disableCoverage(version, os, pipe));
        default:
            return '';
    }
}
exports.addCoverage = addCoverage;
//# sourceMappingURL=coverage.js.map