"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTools = exports.functionRecord = exports.getData = exports.addWPCLI = exports.addPHPUnitTools = exports.addPhive = exports.addPhing = exports.addPECL = exports.addDevTools = exports.addDeployer = exports.addComposer = exports.addCastor = exports.addBlackfirePlayer = exports.addPackage = exports.addArchive = exports.getPharUrl = exports.getUrl = exports.filterList = exports.getRelease = exports.getVersion = exports.getLatestVersion = exports.getSemverVersion = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const fetch = __importStar(require("./fetch"));
const packagist = __importStar(require("./packagist"));
const utils = __importStar(require("./utils"));
async function getSemverVersion(data) {
    const search = data['version_prefix'] + data['version'];
    const url = `https://api.github.com/repos/${data['repository']}/git/matching-refs/tags%2F${search}.`;
    let github_token = await utils.readEnv('GITHUB_TOKEN');
    const composer_token = await utils.readEnv('COMPOSER_TOKEN');
    if (composer_token && !github_token) {
        github_token = composer_token;
    }
    const response = await fetch.fetch(url, github_token);
    if (response.error || response.data === '[]') {
        data['error'] = response.error ?? `No version found with prefix ${search}.`;
        return data['version'];
    }
    else {
        const refs = JSON.parse(response['data']).reverse();
        const ref = refs.find((i) => /.*\d+.\d+.\d+$/.test(i['ref']));
        const tag = (ref || refs[0])['ref'].split('/').pop();
        return tag.replace(/^v(\d)/, '$1');
    }
}
exports.getSemverVersion = getSemverVersion;
async function getLatestVersion(data) {
    if (!data['version'] && data['fetch_latest'] === 'false') {
        return 'latest';
    }
    const resp = await fetch.fetch(`${data['github']}/${data['repository']}/releases.atom`);
    if (resp['data']) {
        const releases = [
            ...resp['data'].matchAll(/releases\/tag\/([a-zA-Z]*)?(\d+.\d+.\d+)"/g)
        ].map(match => match[2]);
        return (releases
            .sort((a, b) => a.localeCompare(b, undefined, { numeric: true }))
            .pop() || 'latest');
    }
    return 'latest';
}
exports.getLatestVersion = getLatestVersion;
async function getVersion(version, data) {
    const semver_regex = /^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$/;
    const composer_regex = /^composer:(stable|preview|snapshot|[1|2])$/;
    const constraint_regex = /[><=^~]+.*/;
    const major_minor_regex = /^\d+(\.\d+)?$/;
    data['version'] = version.replace(/v?(\d)/, '$1').replace(/\.x/, '');
    switch (true) {
        case composer_regex.test(data['release']):
        case semver_regex.test(data['version']):
        case constraint_regex.test(data['version']) && data['type'] === 'composer':
            return data['version'];
        case major_minor_regex.test(data['version']) && data['type'] === 'composer':
            data['release'] = `${data['tool']}:${data['version']}.*`;
            return `${data['version']}.*`;
        case data['repository'] && major_minor_regex.test(data['version']):
            return await getSemverVersion(data);
        default:
            return data['version'].replace(/[><=^~]*/, '');
    }
}
exports.getVersion = getVersion;
async function getRelease(release, data) {
    release = release.includes('/') ? release.split('/')[1] : release;
    return release.includes(':')
        ? [data['tool'], release.split(':')[1]].join(':')
        : data['tool'];
}
exports.getRelease = getRelease;
async function filterList(tools_list) {
    const regex_any = /^composer($|:.*)/;
    const regex_valid = /^composer:?($|preview$|snapshot$|v?\d+(\.\d+)?$|v?\d+\.\d+\.\d+[\w-]*$)/;
    const matches = tools_list.filter(tool => regex_valid.test(tool));
    let composer = 'composer';
    tools_list = tools_list.filter(tool => !regex_any.test(tool));
    switch (true) {
        case matches[0] == undefined:
            break;
        default:
            composer = matches[matches.length - 1].replace(/v(\d\S*)/, '$1');
            break;
    }
    tools_list.unshift(composer);
    return tools_list;
}
exports.filterList = filterList;
async function getUrl(data) {
    if (data['version'] === 'latest') {
        return [
            data['domain'],
            data['repository'],
            data['prefix'],
            data['version'],
            data['verb'],
            data['tool'] + data['extension']
        ]
            .filter(Boolean)
            .join('/');
    }
    else {
        return [
            data['domain'],
            data['repository'],
            data['prefix'],
            data['verb'],
            data['version_prefix'] + data['version'],
            data['tool'] + data['extension']
        ]
            .filter(Boolean)
            .join('/');
    }
}
exports.getUrl = getUrl;
async function getPharUrl(data) {
    if (data['version'] === 'latest') {
        return data['domain'] + '/' + data['tool'] + '.phar';
    }
    else {
        return (data['domain'] +
            '/' +
            data['tool'] +
            '-' +
            data['version_prefix'] +
            data['version'] +
            '.phar');
    }
}
exports.getPharUrl = getPharUrl;
async function addArchive(data) {
    return ((await utils.getCommand(data['os'], 'tool')) +
        (await utils.joins(data['url'], data['tool'], data['version_parameter'])));
}
exports.addArchive = addArchive;
async function addPackage(data) {
    const command = await utils.getCommand(data['os'], 'composer_tool');
    const parts = data['repository'].split('/');
    const args = await utils.joins(parts[1], data['release'], parts[0] + '/', data['scope']);
    return command + args;
}
exports.addPackage = addPackage;
async function addBlackfirePlayer(data) {
    switch (data['os']) {
        case 'win32':
            return await utils.addLog('$cross', data['tool'], data['tool'] + ' is not a windows tool', 'win32');
        default:
            if (data['version'] == 'latest') {
                if (/5\.[5-6]|7\.0/.test(data['php_version'])) {
                    data['version'] = '1.9.3';
                }
                else if (/7\.[1-4]|8\.0/.test(data['php_version'])) {
                    data['version'] = '1.22.0';
                }
            }
            data['url'] = await getPharUrl(data);
            return addArchive(data);
    }
}
exports.addBlackfirePlayer = addBlackfirePlayer;
async function addCastor(data) {
    data['tool'] = 'castor.' + data['os'].replace('win32', 'windows') + '-amd64';
    data['url'] = await getUrl(data);
    data['tool'] = 'castor';
    data['version_parameter'] = fs_1.default.existsSync('castor.php')
        ? data['version_parameter']
        : '';
    return await addArchive(data);
}
exports.addCastor = addCastor;
async function addComposer(data) {
    const channel = data['version'].replace('latest', 'stable');
    const github = data['github'];
    const getcomposer = data['domain'];
    const cds = 'https://dl.cloudsmith.io';
    const filename = `composer-${data['php_version']}-${channel}.phar`;
    const releases_url = `${github}/shivammathur/composer-cache/releases/latest/download/${filename}`;
    const cds_url = `${cds}/public/shivammathur/composer-cache/raw/files/${filename}`;
    const lts_url = `${getcomposer}/download/latest-2.2.x/composer.phar`;
    const is_lts = /^5\.[3-6]$|^7\.[0-1]$/.test(data['php_version']);
    const version_source_url = `${getcomposer}/composer-${channel}.phar`;
    let cache_url = `${releases_url},${cds_url}`;
    let source_url = `${getcomposer}/composer.phar`;
    switch (true) {
        case /^snapshot$/.test(channel):
            source_url = is_lts ? lts_url : source_url;
            break;
        case /^preview$|^2$/.test(channel):
            source_url = is_lts ? lts_url : version_source_url;
            break;
        case /^1$/.test(channel):
            source_url = version_source_url;
            break;
        case /^\d+\.\d+\.\d+[\w-]*$/.test(data['version']):
            cache_url = `${github}/${data['repository']}/releases/download/${data['version']}/composer.phar`;
            source_url = version_source_url;
            break;
        default:
            source_url = is_lts ? lts_url : version_source_url;
    }
    const use_cache = (await utils.readEnv('NO_TOOLS_CACHE')) !== 'true';
    data['url'] = use_cache ? `${cache_url},${source_url}` : source_url;
    data['version_parameter'] = data['version'];
    return await addArchive(data);
}
exports.addComposer = addComposer;
async function addDeployer(data) {
    if (data['version'] === 'latest') {
        data['url'] = data['domain'] + '/deployer.phar';
    }
    else {
        const manifest = await fetch.fetch('https://deployer.org/manifest.json');
        const version_data = JSON.parse(manifest.data);
        const version_key = Object.keys(version_data).find((key) => {
            return version_data[key]['version'] === data['version'];
        });
        if (version_key) {
            data['url'] = version_data[version_key]['url'];
        }
        else {
            return await utils.addLog('$cross', 'deployer', 'Version missing in deployer manifest', data['os']);
        }
    }
    return await addArchive(data);
}
exports.addDeployer = addDeployer;
async function addDevTools(data) {
    switch (data['os']) {
        case 'linux':
        case 'darwin':
            return 'add_devtools ' + data['tool'];
        case 'win32':
            return await utils.addLog('$tick', data['tool'], data['tool'] + ' is not a windows tool', 'win32');
        default:
            return await utils.log('Platform ' + data['os'] + ' is not supported', data['os'], 'error');
    }
}
exports.addDevTools = addDevTools;
async function addPECL(data) {
    return await utils.getCommand(data['os'], 'pecl');
}
exports.addPECL = addPECL;
async function addPhing(data) {
    data['url'] =
        data['domain'] + '/get/phing-' + data['version'] + data['extension'];
    if (data['version'] != 'latest') {
        [data['prefix'], data['verb']] = ['releases', 'download'];
        data['domain'] = data['github'];
        data['extension'] = '-' + data['version'] + data['extension'];
        data['url'] += ',' + (await getUrl(data));
    }
    return await addArchive(data);
}
exports.addPhing = addPhing;
async function addPhive(data) {
    switch (true) {
        case /5\.[3-5]/.test(data['php_version']):
            return await utils.addLog('$cross', 'phive', 'Phive is not supported on PHP ' + data['php_version'], data['os']);
        case /5\.6|7\.0/.test(data['php_version']):
            data['version'] = '0.12.1';
            break;
        case /7\.1/.test(data['php_version']):
            data['version'] = '0.13.5';
            break;
        case /7\.2/.test(data['php_version']):
            data['version'] = '0.14.5';
            break;
        case /^latest$/.test(data['version']):
            data['version'] = await getLatestVersion(data);
            break;
    }
    data['extension'] = '-' + data['version'] + data['extension'];
    data['url'] = await getUrl(data);
    return await addArchive(data);
}
exports.addPhive = addPhive;
async function addPHPUnitTools(data) {
    if (data['version'] === 'latest') {
        data['version'] =
            (await packagist.search(data['packagist'], data['php_version'])) ??
                'latest';
    }
    data['url'] = await getPharUrl(data);
    return await addArchive(data);
}
exports.addPHPUnitTools = addPHPUnitTools;
async function addWPCLI(data) {
    if (data['version'] === 'latest') {
        data['uri'] = 'wp-cli/builds/blob/gh-pages/phar/wp-cli.phar?raw=true';
        data['url'] = [data['domain'], data['uri']].join('/');
    }
    else {
        data['extension'] = '-' + data['version'] + data['extension'];
        data['url'] = await getUrl(data);
    }
    return await addArchive(data);
}
exports.addWPCLI = addWPCLI;
async function getData(release, php_version, os) {
    const json_file_path = path_1.default.join(__dirname, '../src/configs/tools.json');
    const json_file = fs_1.default.readFileSync(json_file_path, 'utf8');
    const json_objects = JSON.parse(json_file);
    release = release.replace(/\s+/g, '');
    const parts = release.split(':');
    const tool = parts[0];
    const version = parts[1];
    let data;
    if (Object.keys(json_objects).includes(tool)) {
        data = json_objects[tool];
        data['tool'] = tool;
    }
    else {
        const key = Object.keys(json_objects).find((key) => {
            return json_objects[key]['alias'] == tool;
        });
        if (key) {
            data = json_objects[key];
            data['tool'] = key;
        }
        else {
            data = {
                tool: tool.split('/')[1],
                repository: tool,
                type: 'composer'
            };
            data = !tool.includes('/') ? { tool: tool } : data;
        }
    }
    data['github'] = 'https://github.com';
    data['domain'] ??= data['github'];
    data['extension'] ??= '.phar';
    data['os'] = os;
    data['php_version'] = php_version;
    data['packagist'] ??= data['repository'];
    data['prefix'] = data['github'] === data['domain'] ? 'releases' : '';
    data['verb'] = data['github'] === data['domain'] ? 'download' : '';
    data['fetch_latest'] ??= 'false';
    data['scope'] ??= 'global';
    data['version_parameter'] = JSON.stringify(data['version_parameter']) || '';
    data['version_prefix'] ??= '';
    data['release'] = await getRelease(release, data);
    data['version'] = version
        ? await getVersion(version, data)
        : await getLatestVersion(data);
    return data;
}
exports.getData = getData;
exports.functionRecord = {
    castor: addCastor,
    composer: addComposer,
    deployer: addDeployer,
    dev_tools: addDevTools,
    phive: addPhive,
    blackfire_player: addBlackfirePlayer,
    pecl: addPECL,
    phing: addPhing,
    phpunit: addPHPUnitTools,
    phpcpd: addPHPUnitTools,
    wp_cli: addWPCLI
};
async function addTools(tools_csv, php_version, os) {
    let script = '\n';
    if (tools_csv === 'none') {
        return '';
    }
    else {
        script += await utils.stepLog('Setup Tools', os);
    }
    const tools_list = await filterList(await utils.CSVArray(tools_csv));
    await utils.asyncForEach(tools_list, async function (release) {
        const data = await getData(release, php_version, os);
        script += '\n';
        switch (true) {
            case data['error'] !== undefined:
                script += await utils.addLog('$cross', data['tool'], data['error'], data['os']);
                break;
            case 'phar' === data['type']:
                data['url'] = await getUrl(data);
                script += await addArchive(data);
                break;
            case 'composer' === data['type']:
                script += await addPackage(data);
                break;
            case 'custom-package' === data['type']:
                script += await utils.customPackage(data['tool'].split('-')[0], 'tools', data['version'], data['os']);
                break;
            case 'custom-function' === data['type']:
                script += await exports.functionRecord[data['function']](data);
                break;
            case /^none$/.test(data['tool']):
                break;
            default:
                script += await utils.addLog('$cross', data['tool'], 'Tool ' + data['tool'] + ' is not supported', data['os']);
                break;
        }
    });
    return script;
}
exports.addTools = addTools;
//# sourceMappingURL=tools.js.map