"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setVariable = exports.readPHPVersion = exports.parseExtensionSource = exports.customPackage = exports.scriptTool = exports.scriptExtension = exports.joins = exports.getCommand = exports.getUnsupportedLog = exports.suppressOutput = exports.getExtensionPrefix = exports.CSVArray = exports.extensionArray = exports.addLog = exports.stepLog = exports.log = exports.color = exports.asyncForEach = exports.parseIniFile = exports.parseVersion = exports.getManifestURL = exports.getInput = exports.readEnv = void 0;
const fs_1 = __importDefault(require("fs"));
const path = __importStar(require("path"));
const core = __importStar(require("@actions/core"));
const fetch = __importStar(require("./fetch"));
async function readEnv(property) {
    const property_lc = property.toLowerCase();
    const property_uc = property.toUpperCase();
    return (process.env[property] ||
        process.env[property_lc] ||
        process.env[property_uc] ||
        process.env[property_lc.replace('_', '-')] ||
        process.env[property_uc.replace('_', '-')] ||
        '');
}
exports.readEnv = readEnv;
async function getInput(name, mandatory) {
    const input = core.getInput(name);
    const env_input = await readEnv(name);
    switch (true) {
        case input != '':
            return input;
        case input == '' && env_input != '':
            return env_input;
        case input == '' && env_input == '' && mandatory:
            throw new Error(`Input required and not supplied: ${name}`);
        default:
            return '';
    }
}
exports.getInput = getInput;
async function getManifestURL() {
    return 'https://raw.githubusercontent.com/shivammathur/setup-php/develop/src/configs/php-versions.json';
}
exports.getManifestURL = getManifestURL;
async function parseVersion(version) {
    switch (true) {
        case /^(latest|nightly|\d+\.x)$/.test(version):
            return JSON.parse((await fetch.fetch(await getManifestURL()))['data'])[version];
        default:
            switch (true) {
                case version.length > 1:
                    return version.slice(0, 3);
                default:
                    return version + '.0';
            }
    }
}
exports.parseVersion = parseVersion;
async function parseIniFile(ini_file) {
    switch (true) {
        case /^(production|development|none)$/.test(ini_file):
            return ini_file;
        case /php\.ini-(production|development)$/.test(ini_file):
            return ini_file.split('-')[1];
        default:
            return 'production';
    }
}
exports.parseIniFile = parseIniFile;
async function asyncForEach(array, callback) {
    for (let index = 0; index < array.length; index++) {
        await callback(array[index], index, array);
    }
}
exports.asyncForEach = asyncForEach;
async function color(type) {
    switch (type) {
        case 'error':
            return '31';
        default:
        case 'success':
            return '32';
        case 'warning':
            return '33';
    }
}
exports.color = color;
async function log(message, os, log_type) {
    switch (os) {
        case 'win32':
            return ('printf "\\033[' +
                (await color(log_type)) +
                ';1m' +
                message +
                ' \\033[0m"');
        case 'linux':
        case 'darwin':
        default:
            return ('echo "\\033[' + (await color(log_type)) + ';1m' + message + '\\033[0m"');
    }
}
exports.log = log;
async function stepLog(message, os) {
    switch (os) {
        case 'win32':
            return 'Step-Log "' + message + '"';
        case 'linux':
        case 'darwin':
            return 'step_log "' + message + '"';
        default:
            return await log('Platform ' + os + ' is not supported', os, 'error');
    }
}
exports.stepLog = stepLog;
async function addLog(mark, subject, message, os) {
    switch (os) {
        case 'win32':
            return 'Add-Log "' + mark + '" "' + subject + '" "' + message + '"';
        case 'linux':
        case 'darwin':
            return 'add_log "' + mark + '" "' + subject + '" "' + message + '"';
        default:
            return await log('Platform ' + os + ' is not supported', os, 'error');
    }
}
exports.addLog = addLog;
async function extensionArray(extension_csv) {
    switch (extension_csv) {
        case '':
        case ' ':
            return [];
        default:
            return [
                extension_csv.match(/(^|,\s?)none(\s?,|$)/) ? 'none' : '',
                ...extension_csv
                    .split(',')
                    .map(function (extension) {
                    if (/.+-.+\/.+@.+/.test(extension)) {
                        return extension;
                    }
                    return extension
                        .trim()
                        .toLowerCase()
                        .replace(/^(:)?(php[-_]|none|zend )|(-[^-]*)-/, '$1$3');
                })
            ].filter(Boolean);
    }
}
exports.extensionArray = extensionArray;
async function CSVArray(values_csv) {
    switch (values_csv) {
        case '':
        case ' ':
            return [];
        default:
            return values_csv
                .split(/,(?=(?:(?:[^"']*["']){2})*[^"']*$)/)
                .map(function (value) {
                return value
                    .trim()
                    .replace(/^["']|["']$|(?<==)["']/g, '')
                    .replace(/=(((?!E_).)*[?{}|&~![()^]+((?!E_).)+)/, "='$1'")
                    .replace(/=(.*?)(=.*)/, "='$1$2'")
                    .replace(/:\s*["'](.*?)/g, ':$1');
            })
                .filter(Boolean);
    }
}
exports.CSVArray = CSVArray;
async function getExtensionPrefix(extension) {
    switch (true) {
        default:
            return 'extension';
        case /xdebug([2-3])?$|opcache|ioncube|eaccelerator/.test(extension):
            return 'zend_extension';
    }
}
exports.getExtensionPrefix = getExtensionPrefix;
async function suppressOutput(os) {
    switch (os) {
        case 'win32':
            return ' >$null 2>&1';
        case 'linux':
        case 'darwin':
            return ' >/dev/null 2>&1';
        default:
            return await log('Platform ' + os + ' is not supported', os, 'error');
    }
}
exports.suppressOutput = suppressOutput;
async function getUnsupportedLog(extension, version, os) {
    return ('\n' +
        (await addLog('$cross', extension, [extension, 'is not supported on PHP', version].join(' '), os)) +
        '\n');
}
exports.getUnsupportedLog = getUnsupportedLog;
async function getCommand(os, suffix) {
    switch (os) {
        case 'linux':
        case 'darwin':
            return 'add_' + suffix + ' ';
        case 'win32':
            return ('Add-' +
                suffix
                    .split('_')
                    .map((part) => part.charAt(0).toUpperCase() + part.slice(1))
                    .join('') +
                ' ');
        default:
            return await log('Platform ' + os + ' is not supported', os, 'error');
    }
}
exports.getCommand = getCommand;
async function joins(...str) {
    return [...str].join(' ');
}
exports.joins = joins;
async function scriptExtension(os) {
    switch (os) {
        case 'win32':
            return '.ps1';
        case 'linux':
        case 'darwin':
            return '.sh';
        default:
            return await log('Platform ' + os + ' is not supported', os, 'error');
    }
}
exports.scriptExtension = scriptExtension;
async function scriptTool(os) {
    switch (os) {
        case 'win32':
            return 'pwsh ';
        case 'linux':
        case 'darwin':
            return 'bash ';
        default:
            return await log('Platform ' + os + ' is not supported', os, 'error');
    }
}
exports.scriptTool = scriptTool;
async function customPackage(pkg, type, version, os) {
    const pkg_name = pkg.replace(/\d+|(pdo|pecl)[_-]/, '');
    const script_extension = await scriptExtension(os);
    const script = path.join(__dirname, '../src/scripts/' + type + '/' + pkg_name + script_extension);
    const command = await getCommand(os, pkg_name);
    return '\n. ' + script + '\n' + command + version;
}
exports.customPackage = customPackage;
async function parseExtensionSource(extension, prefix) {
    const regex = /(\w+)-(\w+:\/\/.{1,253}(?:[.:][^:/\s]{2,63})+\/)?([\w.-]+)\/([\w.-]+)@(.+)/;
    const matches = regex.exec(extension);
    matches[2] = matches[2] ? matches[2].slice(0, -1) : 'https://github.com';
    return await joins('\nadd_extension_from_source', ...matches.splice(1, matches.length), prefix);
}
exports.parseExtensionSource = parseExtensionSource;
async function readPHPVersion() {
    const version = await getInput('php-version', false);
    if (version) {
        return version;
    }
    const versionFile = (await getInput('php-version-file', false)) || '.php-version';
    if (fs_1.default.existsSync(versionFile)) {
        return fs_1.default.readFileSync(versionFile, 'utf8').replace(/[\r\n]/g, '');
    }
    else if (versionFile !== '.php-version') {
        throw new Error(`Could not find '${versionFile}' file.`);
    }
    const composerLock = 'composer.lock';
    if (fs_1.default.existsSync(composerLock)) {
        const lockFileContents = JSON.parse(fs_1.default.readFileSync(composerLock, 'utf8'));
        if (lockFileContents['platform-overrides'] &&
            lockFileContents['platform-overrides']['php']) {
            return lockFileContents['platform-overrides']['php'];
        }
    }
    const composerJson = 'composer.json';
    if (fs_1.default.existsSync(composerJson)) {
        const composerFileContents = JSON.parse(fs_1.default.readFileSync(composerJson, 'utf8'));
        if (composerFileContents['config'] &&
            composerFileContents['config']['platform'] &&
            composerFileContents['config']['platform']['php']) {
            return composerFileContents['config']['platform']['php'];
        }
    }
    return 'latest';
}
exports.readPHPVersion = readPHPVersion;
async function setVariable(variable, command, os) {
    switch (os) {
        case 'win32':
            return '\n$' + variable + ' = ' + command + '\n';
        case 'linux':
        case 'darwin':
        default:
            return '\n' + variable + '="$(' + command + ')"\n';
    }
}
exports.setVariable = setVariable;
//# sourceMappingURL=utils.js.map